/*-
 * Copyright (C)2024 @BABOLO http://www.babolo.ru/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ident "@(#) Copyright (C)2024 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: mod_authn_any.c,v 1.1 2024/03/02 23:58:41 babolo Exp $"

#include <apr_strings.h>
#include <apr_md5.h>
#include <ap_config.h>
#include <ap_provider.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <httpd.h>
#include <http_config.h>
#include <http_core.h>
#include <http_log.h>
#include <http_protocol.h>
#include <http_request.h>
#include <mod_auth.h>

#define INDTEX          "mod_authn_any@BABOLO "
#define MER             APLOG_MARK, APLOG_ERR, 0, r, INDTEX
#define MCO             f, l, m, APLOG_ERR, 0, r, INDTEX
#define MT              "Auth_Any_"

typedef struct {
     int        testenv;
} authn_config_rec;

module AP_MODULE_DECLARE_DATA authn_any_module;

static const command_rec authn_any_cmds[] =
{   AP_INIT_TAKE1( MT "TestEnv"
                 , ap_set_int_slot
                 , (void *)APR_OFFSETOF(authn_config_rec, testenv)
                 , OR_AUTHCFG
                 , "Include TEST_* variables into environment"
                 )
,   {NULL}
};

static void
testset(request_rec *r) {
    if  (!!r->connection) {
        conn_rec *c = r->connection;

        if  (!!c->base_server) {
            server_rec *s = c->base_server;

            if  (!!s->defn_name)
                apr_table_setn(r->subprocess_env, "TEST_BDEFNAME", s->defn_name);
            if  (!!s->server_scheme)
                apr_table_setn(r->subprocess_env, "TEST_BSCHEME", s->server_scheme);
        }
        if  (!!c->client_ip)   apr_table_setn(r->subprocess_env, "TEST_CIP"  , c->client_ip  );
        if  (!!c->remote_host) apr_table_setn(r->subprocess_env, "TEST_RHOST", c->remote_host);
        if  (!!c->local_ip)    apr_table_setn(r->subprocess_env, "TEST_LIP"  , c->local_ip   );
        if  (!!c->local_host)  apr_table_setn(r->subprocess_env, "TEST_LHOST", c->local_host );
    }
    if  (!!r->server) {
        server_rec *s = r->server;

        if  (!!s->defn_name)     apr_table_setn(r->subprocess_env, "TEST_DEFNAME", s->defn_name    );
        if  (!!s->server_scheme) apr_table_setn(r->subprocess_env, "TEST_SCHEME" , s->server_scheme);
    }
    if  (!!r->uri)            apr_table_setn(r->subprocess_env, "TEST_URI"  , r->uri           );
    if  (!!r->args)           apr_table_setn(r->subprocess_env, "TEST_ARGS" , r->args          );
    if  (!!r->protocol)       apr_table_setn(r->subprocess_env, "TEST_PROTO", r->protocol      );
    if  (!!r->the_request)    apr_table_setn(r->subprocess_env, "TEST_REQ"  , r->the_request   );
    if  (!!r->hostname)       apr_table_setn(r->subprocess_env, "TEST_HOST" , r->hostname      );
    if  (!!r->method)         apr_table_setn(r->subprocess_env, "TEST_METH" , r->method        );
    if  (!!r->user)           apr_table_setn(r->subprocess_env, "TEST_USER" , r->user          );
    if  (!!r->unparsed_uri)   apr_table_setn(r->subprocess_env, "TEST_UURI" , r->unparsed_uri  );
    if  (!!r->path_info)      apr_table_setn(r->subprocess_env, "TEST_PATH" , r->path_info     );
    if  (!!r->useragent_ip)   apr_table_setn(r->subprocess_env, "TEST_UIP"  , r->useragent_ip  );
    if  (!!r->useragent_host) apr_table_setn(r->subprocess_env, "TEST_UHOST", r->useragent_host);
    if  (!!ap_http_scheme(r)) apr_table_setn(r->subprocess_env, "TEST_RSCH" , ap_http_scheme(r));
}

static void *
create_authn_any_dir_config(apr_pool_t *p, char *d) {
    authn_config_rec *conf;

    conf = apr_palloc(p, sizeof(authn_config_rec));
    bzero(conf, sizeof(authn_config_rec));
    return(conf);
}

static authn_status
check_password(request_rec *r, const char *user, const char *password) {
    authn_config_rec    *conf   = ap_get_module_config(r->per_dir_config, &authn_any_module);

    apr_table_setn(r->subprocess_env, "ANYPASS", !password ? "" : password);
    if  (!!conf->testenv) testset(r);
    return(AUTH_GRANTED);
}

static authn_status
get_realm_hash(request_rec *r, const char *user, const char *realm, char **rethash) {
    char             *any_hash = NULL;
    authn_config_rec *conf        = ap_get_module_config(r->per_dir_config, &authn_any_module);



    (void)conf;
    if (!any_hash) return(AUTH_USER_NOT_FOUND);
    *rethash = any_hash;
    return(AUTH_USER_FOUND);
}

static const authn_provider authn_any_provider =
{   &check_password
,   &get_realm_hash
};

static void
register_hooks(apr_pool_t *p) {
    ap_register_auth_provider( p
                             , AUTHN_PROVIDER_GROUP
                             , "any"
                             , AUTHN_PROVIDER_VERSION
                             , &authn_any_provider
                             , AP_AUTH_INTERNAL_PER_CONF
                             );
}

AP_DECLARE_MODULE(authn_any) =
{   STANDARD20_MODULE_STUFF
,   create_authn_any_dir_config     /* dir config creater */
,   NULL                            /* dir merger --- default is to override */
,   NULL                            /* server config */
,   NULL                            /* merge server config */
,   authn_any_cmds                  /* command apr_table_t */
,   register_hooks                  /* register hooks */
};
